require 'matrix'

class Vector
  def []=(i,x)
    @elements[i]=x
  end
  def Vector.test()
    v1 = Vector[1,2,3]
    v2 = [1,2,2]
    v1.inner_product(v2)    
  end
end

class Matrix
  def []=(i,j,x)
    @rows[i][j]=x
  end
  def times_vec(v)
    return row_vectors().collect{|row_vec| row_vec.inner_product(v)}
  end
  def Matrix.test()
    m = Matrix[[1,2],[3,4]]
    v = [1,1]
    m.times_vec(v)
  end  
end

class BasicStats
  
public
  def BasicStats.average(data)
    ave = 0.0
    data.each do |a|
      ave += a  
    end
    return ave/data.length
  end

  def BasicStats.median(data)
    med = 0
    sorted_data = data.sort{|a,b| a <=> b}
    len = sorted_data.length
    if(len % 2 == 0)
      return (sorted_data[(len-1)/2]+sorted_data[(len+1)/2])/2.0
    else
      return sorted_data[len/2]
    end
  end
  
  def BasicStats.sd_hat_unbiased(arr)
    num = arr.length
    if(arr.length < 2)
      return 0
    else
      x_bar = 0.0
      x_squared_bar = 0.0
      arr.each do |e|
        x_bar += e
        x_squared_bar += e*e
      end
      x_squared_bar = x_squared_bar / (num - 1.0)
      x_bar = x_bar / num
      x_bar_squared = x_bar * x_bar
      return Math.sqrt(x_squared_bar - x_bar_squared*num*(1.0 / (num - 1)) )
    end
  end
  
  def BasicStats.sd_hat_MLE(arr)
    num = arr.length
    if(arr.length < 2)
      return 0
    else
      x_bar = 0.0
      x_squared_bar = 0.0
      arr.each do |e|
        x_bar += e
        x_squared_bar += e*e
      end
      x_squared_bar = x_squared_bar * (1.0 / num)
      x_bar = x_bar *(1.0 / num)
      x_bar_squared = x_bar * x_bar
      return Math.sqrt(x_squared_bar - x_bar_squared)
    end
  end
  
  def BasicStats.get_range(gene_expr_values_arr)
    return [gene_expr_values_arr.min,gene_expr_values_arr.max]
  end

  def BasicStats.z_transform_single_Spearman_Corr(corr, sample_size)
    small = 0.001
    if(corr > 0)
      corr -= small
    else
      corr += small 
    end
    
    z = -0.5 * (Math.sqrt(sample_size - 3) * Math.log( (1-corr)/(1+corr) ) ) 
    return z
#    sd = BasicStats.sd_hat_MLE(z)
#    return z.collect{|a| a / sd}
  end


  def BasicStats.z_transform_Spearman_Corr(s, sample_size)
    z = s.collect{|corr| -0.5 * (Math.sqrt(sample_size - 3) * Math.log( (1-corr)/(1+corr) ) ) }
    return z
    sd = BasicStats.sd_hat_MLE(z)
    return z.collect{|a| a / sd}
  end

  def BasicStats.get_ranks(x)
    i = 0
    x_hash = {}
    x.each do |e|
      i += 1
      x_hash[i] = e
    end
    i = 0
    x_sorted = x_hash.sort{|a,b| b[1] <=> a[1]}.map {|e| i+=1;e[1] = i;e}.sort{|a1,b1| a1[0] <=> b1[0]}  .collect{|a| a[1]}
    return x_sorted
  end
  
  
  def BasicStats.SpearmanRankCorr(x,y)
    x_ranks = get_ranks(x)
    y_ranks = get_ranks(y)
    return BasicStats.PearsonCorr(x_ranks,y_ranks)
  end
  
  
  def BasicStats.PearsonCorr(x,y)
    res = 0
    x_ave = BasicStats.average(x)
    y_ave = BasicStats.average(y)
    x_sd  = BasicStats.sd_hat_unbiased(x)
    y_sd  = BasicStats.sd_hat_unbiased(y)
    (0...x.length).each do |i|
      z_x = ((x[i] - x_ave) / x_sd)
      z_y = ((y[i] - y_ave) / y_sd)
      res += z_x * z_y
    end
    res = res / (x.length - 1)
#    x = [1,1,0.5]
#    y = [1,0.9,0.6]
#    res_wiki = BasicStats.PearsonCorr_wikipedia(x,y)
#    print x.join(" "),"\n", y.join(" "),"\n"
#    print "mu = ", BasicStats.sd_hat_unbiased(x), "\tsd = ", BasicStats.average(x), "\n"
#    print "mu = ", BasicStats.sd_hat_unbiased(y), "\tsd = ", BasicStats.average(y), "\n"
#    print "res_wiki = ", res_wiki, "\tres = ", res
#    gets
    return res
  end
  
 def BasicStats.PearsonCorr_wikipedia(x,y)
     sum_sq_x = 0.0
     sum_sq_y = 0.0
     sum_coproduct = 0.0
     mean_x = x[0].to_f
     mean_y = y[0].to_f
     n = x.length
     (2...n).each do |ind|
       i = ind - 1
       sweep = (i - 1.0) / i.to_f
       delta_x = x[ind] - mean_x
       delta_y = y[ind] - mean_y
       sum_sq_x += delta_x * delta_x * sweep
       print "sum_sq_x = ", sum_sq_x, "\n"
       sum_sq_y += delta_y * delta_y * sweep
       sum_coproduct += delta_x * delta_y * sweep
       mean_x += delta_x / i
       mean_y += delta_y / i
     end
     pop_sd_x = Math.sqrt( sum_sq_x/n )
     pop_sd_y = Math.sqrt( sum_sq_y/n )
     
     print "pop_sd_x = ", pop_sd_x,"\n" 
     print "pop_sd_y = ", pop_sd_y,"\n"

     cov_x_y = sum_coproduct/n
     correlation = cov_x_y / (pop_sd_x * pop_sd_y)
     return correlation
  end
  


# end class
end
