#!/usr/bin/perl

$infile = $ARGV[0]; # input file of genomic locations in bed format
$twobitfile = $ARGV[1];
$outfile = $ARGV[2];

$tmpfile = "$infile.tmp.fa";
$bedfile = "$infile.twoBitGrab";

open(IN, $infile);
open(OUT, ">$outfile");
open(BED, ">$bedfile");

while (<IN>) {
    $line = $_;
    chomp($line);
    @linearr = split(/\t/, $line);

    $chr = $linearr[0];
    $start = $linearr[1];
    $end = $linearr[2];
    $name = $linearr[3];
    $note = $linearr[4];
    $strand = $linearr[5];

    if ($end <= $start) {
	print "WARNING: Sequence for $id not retrieved because end=$end <= start=$start\n";
	next;
    }

    $command = "./twoBitToFa -noMask $twobitfile:$chr:$start-$end $tmpfile";
    $output = `$command 2>&1`;
#    print "$command\n";
#    print "$output\n";

    if ($output =~ /twoBitReadSeqFrag end \(([0-9]+)\) >= seqSize \(([0-9]+)\)/) {
	$maxsize = $2;
	$end = $maxsize;
	$command = "./twoBitToFa -noMask $twobitfile:$chr:$start-$end $tmpfile";
	print "WARNING: truncating $name sequence end to chromosome size $maxsize.\n";
	$output = `$command 2>&1`;
	$note = "truncated";
    }

    open (TMP, "$tmpfile");
    $header = <TMP>;
    $seq = "";
    while (<TMP>) {
	$line = $_;
	chomp($line);
	$seq .= $line;
    }
    close(TMP);

    if ($strand eq "-") {
	# take reverse complement
        $seq = reverse $seq;
        $seq =~ tr/ACGTacgt/TGCAtgca/;
    }

    $outheader = "$name\t$chr\t$start\_$end\t$strand";

    if ($seq ne "") {
	print OUT ">$outheader\n";
	print OUT "$seq\n";
    }
    else {
	print "WARNING: Retrieval for $name failed.\n";
	$note = "failed";
    }

    $bedline = join("\t", $chr, $start, $end, $name, $note, $strand);
    print BED "$bedline\n";

}

close(IN);
close(OUT);
close(BED);

`rm $tmpfile`;
